import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimJob(
        filename_temp='TestModel_MultipleZone',
        ScriptType='mz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_addMultipleZoneSch(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addMultipleZoneSch(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    addMultipleZoneSch_dict = {
        'On 24/7': ['Until: 24:00', '1'],
        'Control type schedule: Always 4': ['Until: 24:00', '4'],
        'Relative humidity setpoint schedule: Always 50.00': ['Until: 24:00', '50'],
        'Heating Fanger comfort setpoint: Always -0.5': ['Until: 24:00', '-0.5'],
        'Cooling Fanger comfort setpoint: Always  0.1': ['Until: 24:00', '0.1'],
        'Zone CO2 setpoint: Always 900ppm': ['Until: 24:00', '900'],
        'Min CO2 concentration: Always 600ppm': ['Until: 24:00', '600'],
        'Generic contaminant setpoint: Always 0.5ppm': ['Until: 24:00', '0.5'],
        'Air distribution effectiveness (always 1)': ['Until: 24:00', '1']
        }
    
    for i in addMultipleZoneSch_dict:
        sch = ([x
                for x
                in idf1.idfobjects['Schedule:Compact']
                if x.Name == i])
        assert sch[0].Name == i
        assert sch[0].Schedule_Type_Limits_Name == "Any Number"
        assert sch[0].Field_1 == 'Through: 12/31'
        assert sch[0].Field_2 == 'For: AllDays'
        assert sch[0].Field_3 == addMultipleZoneSch_dict[i][0]
        assert sch[0].Field_4 == addMultipleZoneSch_dict[i][1]


    sch = ([x
            for x
            in idf1.idfobjects['Schedule:Compact']
            if x.Name == 'VRF Heating Cooling (Northern Hemisphere)'])
    assert sch[0].Name == "VRF Heating Cooling (Northern Hemisphere)"
    assert sch[0].Schedule_Type_Limits_Name == "Any Number"
    assert sch[0].Field_1 == 'Through: 31 Mar'
    assert sch[0].Field_2 == 'For: AllDays'
    assert sch[0].Field_3 == 'Until: 24:00'
    assert sch[0].Field_4 == '0'
    assert sch[0].Field_5 == 'Through: 30 Sep'
    assert sch[0].Field_6 == 'For: AllDays'
    assert sch[0].Field_7 == 'Until: 24:00'
    assert sch[0].Field_8 == '1'
    assert sch[0].Field_9 == 'Through: 31 Dec'
    assert sch[0].Field_10 == 'For: AllDays'
    assert sch[0].Field_11 == 'Until: 24:00'
    assert sch[0].Field_12 == '0'


def test_addCurveObj(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addCurveObj(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')
    
    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'DefaultFanEffRatioCurve'])
    assert curveobj[0].Name == 'DefaultFanEffRatioCurve'
    assert curveobj[0].Coefficient1_Constant == 0.33856828
    assert curveobj[0].Coefficient2_x == 1.72644131
    assert curveobj[0].Coefficient3_x2 == -1.49280132
    assert curveobj[0].Coefficient4_x3 == 0.42776208
    assert curveobj[0].Minimum_Value_of_x == 0.5
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == 0.3
    assert curveobj[0].Maximum_Curve_Output == 1.0
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFTUCoolCapFT'])
    assert curveobj[0].Name == 'VRFTUCoolCapFT'
    assert curveobj[0].Coefficient1_Constant == 0.504547273506488
    assert curveobj[0].Coefficient2_x == 0.0288891279198444
    assert curveobj[0].Coefficient3_x2 == -0.000010819418650677
    assert curveobj[0].Coefficient4_x3 == 0.0000101359395177008
    assert curveobj[0].Minimum_Value_of_x == 0.0
    assert curveobj[0].Maximum_Value_of_x == 50.0
    assert curveobj[0].Minimum_Curve_Output == 0.5
    assert curveobj[0].Maximum_Curve_Output == 1.5
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFTUHeatCapFT'])
    assert curveobj[0].Name == 'VRFTUHeatCapFT'
    assert curveobj[0].Coefficient1_Constant == -0.390708928227928
    assert curveobj[0].Coefficient2_x == 0.261815023760162
    assert curveobj[0].Coefficient3_x2 == -0.0130431603151873
    assert curveobj[0].Coefficient4_x3 == 0.000178131745997821
    assert curveobj[0].Minimum_Value_of_x == 0.0
    assert curveobj[0].Maximum_Value_of_x == 50.0
    assert curveobj[0].Minimum_Curve_Output == 0.5
    assert curveobj[0].Maximum_Curve_Output == 1.5
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFCoolCapFTBoundary'])
    assert curveobj[0].Name == 'VRFCoolCapFTBoundary'
    assert curveobj[0].Coefficient1_Constant == 25.73473775
    assert curveobj[0].Coefficient2_x == -0.03150043
    assert curveobj[0].Coefficient3_x2 == -0.01416595
    assert curveobj[0].Coefficient4_x3 == 0
    assert curveobj[0].Minimum_Value_of_x == 11
    assert curveobj[0].Maximum_Value_of_x == 30
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFCoolEIRFTBoundary'])
    assert curveobj[0].Name == 'VRFCoolEIRFTBoundary'
    assert curveobj[0].Coefficient1_Constant == 25.73473775
    assert curveobj[0].Coefficient2_x == -0.03150043
    assert curveobj[0].Coefficient3_x2 == -0.01416595
    assert curveobj[0].Coefficient4_x3 == 0
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 24
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'CoolingEIRLowPLR'])
    assert curveobj[0].Name == 'CoolingEIRLowPLR'
    assert curveobj[0].Coefficient1_Constant == 0.4628123
    assert curveobj[0].Coefficient2_x == -1.0402406
    assert curveobj[0].Coefficient3_x2 == 2.17490997
    assert curveobj[0].Coefficient4_x3 == -0.5974817
    assert curveobj[0].Minimum_Value_of_x == 0
    assert curveobj[0].Maximum_Value_of_x == 1
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Capacity'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFHeatCapFTBoundary'])
    assert curveobj[0].Name == 'VRFHeatCapFTBoundary'
    assert curveobj[0].Coefficient1_Constant == -7.6000882
    assert curveobj[0].Coefficient2_x == 3.05090016
    assert curveobj[0].Coefficient3_x2 == -0.1162844
    assert curveobj[0].Coefficient4_x3 == 0.0
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'VRFHeatEIRFTBoundary'])
    assert curveobj[0].Name == 'VRFHeatEIRFTBoundary'
    assert curveobj[0].Coefficient1_Constant == -7.6000882
    assert curveobj[0].Coefficient2_x == 3.05090016
    assert curveobj[0].Coefficient3_x2 == -0.1162844
    assert curveobj[0].Coefficient4_x3 == 0.0
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Curve_Output == -20
    assert curveobj[0].Maximum_Curve_Output == 15
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Cubic']
                 if x.Name == 'HeatingEIRLowPLR'])
    assert curveobj[0].Name == 'HeatingEIRLowPLR'
    assert curveobj[0].Coefficient1_Constant == 0.1400093
    assert curveobj[0].Coefficient2_x == 0.6415002
    assert curveobj[0].Coefficient3_x2 == 0.1339047
    assert curveobj[0].Coefficient4_x3 == 0.0845859
    assert curveobj[0].Minimum_Value_of_x == 0
    assert curveobj[0].Maximum_Value_of_x == 1
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Dimensionless'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Exponent']
                 if x.Name == 'DefaultFanPowerRatioCurve'])
    assert curveobj[0].Name == 'DefaultFanPowerRatioCurve'
    assert curveobj[0].Coefficient1_Constant == 0
    assert curveobj[0].Coefficient2_Constant == 1
    assert curveobj[0].Coefficient3_Constant == 3
    assert curveobj[0].Minimum_Value_of_x == 0
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == 0.01
    assert curveobj[0].Maximum_Curve_Output == 1.5
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'DXHtgCoilDefrostEIRFT'])
    assert curveobj[0].Name == 'DXHtgCoilDefrostEIRFT'
    assert curveobj[0].Coefficient1_Constant == 1.0
    assert curveobj[0].Coefficient2_x == 0.0
    assert curveobj[0].Coefficient3_x2 == 0.0
    assert curveobj[0].Coefficient4_y == 0.0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == 0.0
    assert curveobj[0].Maximum_Value_of_x == 50.0
    assert curveobj[0].Minimum_Value_of_y == 0.0
    assert curveobj[0].Maximum_Value_of_y == 50.0
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFCoolCapFT'])
    assert curveobj[0].Name == 'VRFCoolCapFT'
    assert curveobj[0].Coefficient1_Constant == 0.576882692
    assert curveobj[0].Coefficient2_x == 0.017447952
    assert curveobj[0].Coefficient3_x2 == 0.000583269
    assert curveobj[0].Coefficient4_y == -1.76324E-06
    assert curveobj[0].Coefficient5_y2 == -7.474E-09
    assert curveobj[0].Coefficient6_xy == -1.30413E-07
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 24
    assert curveobj[0].Minimum_Value_of_y == -5
    assert curveobj[0].Maximum_Value_of_y == 23
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFCoolCapFTHi'])
    assert curveobj[0].Name == 'VRFCoolCapFTHi'
    assert curveobj[0].Coefficient1_Constant == 0.6867358
    assert curveobj[0].Coefficient2_x == 0.0207631
    assert curveobj[0].Coefficient3_x2 == 0.0005447
    assert curveobj[0].Coefficient4_y == -0.0016218
    assert curveobj[0].Coefficient5_y2 == -4.259E-07
    assert curveobj[0].Coefficient6_xy == -0.0003392
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 24
    assert curveobj[0].Minimum_Value_of_y == 16
    assert curveobj[0].Maximum_Value_of_y == 43
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFCoolEIRFT'])
    assert curveobj[0].Name == 'VRFCoolEIRFT'
    assert curveobj[0].Coefficient1_Constant == 0.989010541
    assert curveobj[0].Coefficient2_x == -0.02347967
    assert curveobj[0].Coefficient3_x2 == 0.000199711
    assert curveobj[0].Coefficient4_y == 0.005968336
    assert curveobj[0].Coefficient5_y2 == -1.0289E-07
    assert curveobj[0].Coefficient6_xy == -0.00015686
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 24
    assert curveobj[0].Minimum_Value_of_y == -5
    assert curveobj[0].Maximum_Value_of_y == 23
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFCoolEIRFTHi'])
    assert curveobj[0].Name == 'VRFCoolEIRFTHi'
    assert curveobj[0].Coefficient1_Constant == 0.14351470
    assert curveobj[0].Coefficient2_x == 0.01860035
    assert curveobj[0].Coefficient3_x2 == -0.0003954
    assert curveobj[0].Coefficient4_y == 0.02485219
    assert curveobj[0].Coefficient5_y2 == 0.00016329
    assert curveobj[0].Coefficient6_xy == -0.0006244
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 24
    assert curveobj[0].Minimum_Value_of_y == 16
    assert curveobj[0].Maximum_Value_of_y == 43
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFHeatCapFT'])
    assert curveobj[0].Name == 'VRFHeatCapFT'
    assert curveobj[0].Coefficient1_Constant == 1.014599599
    assert curveobj[0].Coefficient2_x == -0.002506703
    assert curveobj[0].Coefficient3_x2 == -0.000141599
    assert curveobj[0].Coefficient4_y == 0.026931595
    assert curveobj[0].Coefficient5_y2 == 1.83538E-06
    assert curveobj[0].Coefficient6_xy == -0.000358147
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Value_of_y == -20
    assert curveobj[0].Maximum_Value_of_y == 15
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFHeatCapFTHi'])
    assert curveobj[0].Name == 'VRFHeatCapFTHi'
    assert curveobj[0].Coefficient1_Constant == 1.161134821
    assert curveobj[0].Coefficient2_x == 0.027478868
    assert curveobj[0].Coefficient3_x2 == -0.00168795
    assert curveobj[0].Coefficient4_y == 0.001783378
    assert curveobj[0].Coefficient5_y2 == 2.03208E-06
    assert curveobj[0].Coefficient6_xy == -6.8969E-05
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Value_of_y == -10
    assert curveobj[0].Maximum_Value_of_y == 15
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFHeatEIRFT'])
    assert curveobj[0].Name == 'VRFHeatEIRFT'
    assert curveobj[0].Coefficient1_Constant == 0.87465501
    assert curveobj[0].Coefficient2_x == -0.01319754
    assert curveobj[0].Coefficient3_x2 == 0.00110307
    assert curveobj[0].Coefficient4_y == -0.0133118
    assert curveobj[0].Coefficient5_y2 == 0.00089017
    assert curveobj[0].Coefficient6_xy == -0.00012766
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Value_of_y == -20
    assert curveobj[0].Maximum_Value_of_y == 12
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRFHeatEIRFTHi'])
    assert curveobj[0].Name == 'VRFHeatEIRFTHi'
    assert curveobj[0].Coefficient1_Constant == 2.504005146
    assert curveobj[0].Coefficient2_x == -0.05736767
    assert curveobj[0].Coefficient3_x2 == 4.07336E-05
    assert curveobj[0].Coefficient4_y == -0.12959669
    assert curveobj[0].Coefficient5_y2 == 0.00135839
    assert curveobj[0].Coefficient6_xy == 0.00317047
    assert curveobj[0].Minimum_Value_of_x == 15
    assert curveobj[0].Maximum_Value_of_x == 27
    assert curveobj[0].Minimum_Value_of_y == -10
    assert curveobj[0].Maximum_Value_of_y == 15
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'CoolingLengthCorrectionFactor'])
    assert curveobj[0].Name == 'CoolingLengthCorrectionFactor'
    assert curveobj[0].Coefficient1_Constant == 1.0693794
    assert curveobj[0].Coefficient2_x == -0.0014951
    assert curveobj[0].Coefficient3_x2 == 2.56E-06
    assert curveobj[0].Coefficient4_y == -0.1151104
    assert curveobj[0].Coefficient5_y2 == 0.0511169
    assert curveobj[0].Coefficient6_xy == -0.0004369
    assert curveobj[0].Minimum_Value_of_x == 8
    assert curveobj[0].Maximum_Value_of_x == 175
    assert curveobj[0].Minimum_Value_of_y == 0.5
    assert curveobj[0].Maximum_Value_of_y == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRF Piping Correction Factor for Length in Heating Mode'])
    assert curveobj[0].Name == 'VRF Piping Correction Factor for Length in Heating Mode'
    assert curveobj[0].Coefficient1_Constant == .989916
    assert curveobj[0].Coefficient2_x == .001961
    assert curveobj[0].Coefficient3_x2 == -.000036
    assert curveobj[0].Coefficient4_y == 0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == 7
    assert curveobj[0].Maximum_Value_of_x == 106.5
    assert curveobj[0].Minimum_Value_of_y == 1
    assert curveobj[0].Maximum_Value_of_y == 1
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Distance'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Dimensionless'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRF Heat Recovery Cooling Capacity Modifier'])
    assert curveobj[0].Name == 'VRF Heat Recovery Cooling Capacity Modifier'
    assert curveobj[0].Coefficient1_Constant == .9
    assert curveobj[0].Coefficient2_x == 0
    assert curveobj[0].Coefficient3_x2 == 0
    assert curveobj[0].Coefficient4_y == 0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == -100
    assert curveobj[0].Maximum_Value_of_x == 100
    assert curveobj[0].Minimum_Value_of_y == -100
    assert curveobj[0].Maximum_Value_of_y == 100
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRF Heat Recovery Cooling Energy Modifier'])
    assert curveobj[0].Name == 'VRF Heat Recovery Cooling Energy Modifier'
    assert curveobj[0].Coefficient1_Constant == 1.1
    assert curveobj[0].Coefficient2_x == 0
    assert curveobj[0].Coefficient3_x2 == 0
    assert curveobj[0].Coefficient4_y == 0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == -100
    assert curveobj[0].Maximum_Value_of_x == 100
    assert curveobj[0].Minimum_Value_of_y == -100
    assert curveobj[0].Maximum_Value_of_y == 100
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRF Heat Recovery Heating Capacity Modifier'])
    assert curveobj[0].Name == 'VRF Heat Recovery Heating Capacity Modifier'
    assert curveobj[0].Coefficient1_Constant == .9
    assert curveobj[0].Coefficient2_x == 0
    assert curveobj[0].Coefficient3_x2 == 0
    assert curveobj[0].Coefficient4_y == 0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == -100
    assert curveobj[0].Maximum_Value_of_x == 100
    assert curveobj[0].Minimum_Value_of_y == -100
    assert curveobj[0].Maximum_Value_of_y == 100
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Biquadratic']
                 if x.Name == 'VRF Heat Recovery Heating Energy Modifier'])
    assert curveobj[0].Name == 'VRF Heat Recovery Heating Energy Modifier'
    assert curveobj[0].Coefficient1_Constant == 1.1
    assert curveobj[0].Coefficient2_x == 0
    assert curveobj[0].Coefficient3_x2 == 0
    assert curveobj[0].Coefficient4_y == 0
    assert curveobj[0].Coefficient5_y2 == 0
    assert curveobj[0].Coefficient6_xy == 0
    assert curveobj[0].Minimum_Value_of_x == -100
    assert curveobj[0].Maximum_Value_of_x == 100
    assert curveobj[0].Minimum_Value_of_y == -100
    assert curveobj[0].Maximum_Value_of_y == 100
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == 'Temperature'
    assert curveobj[0].Input_Unit_Type_for_Y == 'Temperature'
    assert curveobj[0].Output_Unit_Type == 'Dimensionless'

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Quadratic']
                 if x.Name == 'VRFACCoolCapFFF'])
    assert curveobj[0].Name == 'VRFACCoolCapFFF'
    assert curveobj[0].Coefficient1_Constant == 0.8
    assert curveobj[0].Coefficient2_x == 0.2
    assert curveobj[0].Coefficient3_x2 == 0.0
    assert curveobj[0].Minimum_Value_of_x == 0.5
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Quadratic']
                 if x.Name == 'CoolingEIRHiPLR'])
    assert curveobj[0].Name == 'CoolingEIRHiPLR'
    assert curveobj[0].Coefficient1_Constant == 1.0
    assert curveobj[0].Coefficient2_x == 0.0
    assert curveobj[0].Coefficient3_x2 == 0.0
    assert curveobj[0].Minimum_Value_of_x == 1.0
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Quadratic']
                 if x.Name == 'VRFCPLFFPLR'])
    assert curveobj[0].Name == 'VRFCPLFFPLR'
    assert curveobj[0].Coefficient1_Constant == 0.85
    assert curveobj[0].Coefficient2_x == 0.15
    assert curveobj[0].Coefficient3_x2 == 0.0
    assert curveobj[0].Minimum_Value_of_x == 0.0
    assert curveobj[0].Maximum_Value_of_x == 1.0
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Quadratic']
                 if x.Name == 'HeatingEIRHiPLR'])
    assert curveobj[0].Name == 'HeatingEIRHiPLR'
    assert curveobj[0].Coefficient1_Constant == 2.4294355
    assert curveobj[0].Coefficient2_x == -2.235887
    assert curveobj[0].Coefficient3_x2 == 0.8064516
    assert curveobj[0].Minimum_Value_of_x == 1.0
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Linear']
                 if x.Name == 'CoolingCombRatio'])
    assert curveobj[0].Name == 'CoolingCombRatio'
    assert curveobj[0].Coefficient1_Constant == 0.618055
    assert curveobj[0].Coefficient2_x == 0.381945
    assert curveobj[0].Minimum_Value_of_x == 1.0
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''

    curveobj = ([x
                 for x
                 in idf1.idfobjects['Curve:Linear']
                 if x.Name == 'HeatingCombRatio'])
    assert curveobj[0].Name == 'HeatingCombRatio'
    assert curveobj[0].Coefficient1_Constant == 0.96034
    assert curveobj[0].Coefficient2_x == 0.03966
    assert curveobj[0].Minimum_Value_of_x == 1.0
    assert curveobj[0].Maximum_Value_of_x == 1.5
    assert curveobj[0].Minimum_Curve_Output == ''
    assert curveobj[0].Maximum_Curve_Output == ''
    assert curveobj[0].Input_Unit_Type_for_X == ''
    assert curveobj[0].Output_Unit_Type == ''


def test_addForscriptSchMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addForscriptSchMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')


    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])

    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in idf1.idfobjects['ZONE']])])

    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['Schedule:Compact']
                if x.Name == "FORSCRIPT_AHST_" + zonename])
        assert obj[0].Name == "FORSCRIPT_AHST_" + zonename
        assert obj[0].Schedule_Type_Limits_Name == "Any Number"
        assert obj[0].Field_1 == 'Through: 12/31'
        assert obj[0].Field_2 == 'For: AllDays'
        assert obj[0].Field_3 == 'Until: 24:00'
        assert obj[0].Field_4 == '20'
        
        obj = ([x
                for x
                in idf1.idfobjects['Schedule:Compact']
                if x.Name == "FORSCRIPT_ACST_" + zonename])
        assert obj[0].Name == "FORSCRIPT_ACST_" + zonename
        assert obj[0].Schedule_Type_Limits_Name == "Any Number"
        assert obj[0].Field_1 == 'Through: 12/31'
        assert obj[0].Field_2 == 'For: AllDays'
        assert obj[0].Field_3 == 'Until: 24:00'
        assert obj[0].Field_4 == '24'
    
    for i in range(len(zonenames_orig)):
        obj = ([x
                for x
                in idf1.idfobjects['ThermostatSetpoint:DualSetpoint']
                if x.Name == zonenames_orig[i]+' Dual SP'])
        assert obj[0].Name == zonenames_orig[i]+' Dual SP'
        assert obj[0].Heating_Setpoint_Temperature_Schedule_Name == "FORSCRIPT_AHST_" + zonenames[i]
        assert obj[0].Cooling_Setpoint_Temperature_Schedule_Name == "FORSCRIPT_ACST_" + zonenames[i]

def test_checkVentIsOn(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.checkVentIsOn(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['Schedule:Compact']
            if x.Name == "Vent_SP_temp"])
    assert obj[0].Name == "Vent_SP_temp"
    assert obj[0].Schedule_Type_Limits_Name == "Any Number"
    assert obj[0].Field_1 == 'Through: 12/31'
    assert obj[0].Field_2 == 'For: AllDays'
    assert obj[0].Field_3 == 'Until: 24:00'
    assert obj[0].Field_4 == '24'

    windowlist = ([x
                   for x
                   in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening']
                   if x.Name.endswith('_Win')])
    
    for window in windowlist:
        assert window.Height_Factor_for_Opening_Factor_1 == 1
        assert window.Start_Height_Factor_for_Opening_Factor_1 == 0
        assert window.Width_Factor_for_Opening_Factor_2 == 1
        assert window.Height_Factor_for_Opening_Factor_2 == 1
        assert window.Start_Height_Factor_for_Opening_Factor_2 == 0
    
    windowlist_2 = ([window
                     for window
                     in idf1.idfobjects['AirflowNetwork:MultiZone:Surface']
                     if window.Surface_Name.endswith('_Win')])
    for window in windowlist_2:
        assert window.Ventilation_Control_Mode == 'Temperature'
        assert window.Ventilation_Control_Zone_Temperature_Setpoint_Schedule_Name == 'Vent_SP_temp'
        assert window.Venting_Availability_Schedule_Name == 'On'

    doorlist = ([door
                 for door
                 in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening']
                 if door.Name.endswith('_Door')])
    for door in doorlist:
        assert door.Width_Factor_for_Opening_Factor_2 == 1

    doorlist_2 = ([door
                   for door
                   in idf1.idfobjects['AirflowNetwork:MultiZone:Surface']
                   if door.Surface_Name.endswith('_Door')])
    for door in doorlist_2:
        assert door.Venting_Availability_Schedule_Name == 'On'



"""Add EnergyPlus objects only for SingleZone Mode."""


def addForscriptSchSingleZone(self, verboseMode: bool = True):
    """Add FORSCRIPT schedules for SingleZone accim."""
    if "FORSCRIPT_AHST" in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("FORSCRIPT_AHST Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="FORSCRIPT_AHST",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 12/31',
            Field_2='For: AllDays',
            Field_3='Until: 24:00,20'
            )
        # todo allow mixed mode for singlezone functions: Name="FORSCRIPT_ACST_"+zonename,
        if verboseMode:
            print("FORSCRIPT_AHST Schedule has been added")
    if "FORSCRIPT_ACST" in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("FORSCRIPT_ACST Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="FORSCRIPT_ACST",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 12/31',
            Field_2='For: AllDays',
            Field_3='Until: 24:00,24'
            )
        if verboseMode:
            print("FORSCRIPT_ACST Schedule has been added")

    for program in [program for program in self.idf1.idfobjects['ThermostatSetpoint:DualSetpoint']]:
        program.Heating_Setpoint_Temperature_Schedule_Name = "FORSCRIPT_AHST"
        program.Cooling_Setpoint_Temperature_Schedule_Name = "FORSCRIPT_ACST"
    if verboseMode:
        print('FORSCRIPT_AHST and FORSCRIPT_ACST has been asigned to all ThermostatSetpoint:DualSetpoint objects')



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""

import pytest
from accim.sim import accim_Main

class test_SingleZone:
    def test_common(self):
        from eppy.modeleditor import IDF

        iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        IDF.setiddname(iddfile)

        idf0 = IDF('TestModel_SingleZone.idf')

        programlist = ([program.Name for program in idf0.idfobjects['EnergyManagementSystem:Program']])
        self.z = accim_Main.accimobj_SingleZone_Ep94('TestModel_SingleZone')

    def test_addEMSProgramsBase(self):

        from eppy.modeleditor import IDF

        self.z.addEMSProgramsBase()
        self.z.saveaccim()

        idf1 = IDF('TestModel_SingleZone_pymod.idf')

        programlist = ([program.Name for program in idf1.idfobjects['EnergyManagementSystem:Program']])
        assert ('SetComfTemp' in programlist) == True

        SetComfTemp = ([program
                        for program
                        in idf1.idfobjects['EnergyManagementSystem:Program']
                        if program.Name == 'SetComfTemp'])

        assert SetComfTemp[0].Program_Line_1 == 'if AdapStand == 1'
        assert SetComfTemp[0].Program_Line_2 == 'set ComfTemp = RMOT*0.33+18.8'
        assert SetComfTemp[0].Program_Line_3 == 'elseif AdapStand == 2'
        assert SetComfTemp[0].Program_Line_4 == 'set ComfTemp = PMOT*0.31+17.8'
        assert SetComfTemp[0].Program_Line_5 == 'endif'

        # for zonename in self.zonenames:

# y = tests_SingleZone()
# y.test_addEMSProgramsBase()



"""
Run the function below to add the ACCIS.

This function transform fixed setpoint temperature
building energy models into adaptive setpoint temperature energy models
by adding the Adaptive Comfort Control Implementation Script (ACCIS)
"""

def addAccis(
        ScriptType: str = None,
        Outputs: str = None,
        EnergyPlus_version: str = None,
        AdapStand: any = None,
        CAT: any = None,
        ComfMod: any = None,
        HVACmode: any = None,
        VentCtrl: any = None,
        VSToffset: any = [0],
        MinOToffset: any = [50],
        MaxWindSpeed: any = [50],
        ASTtol_start: float = 0.1,
        ASTtol_end_input: float = 0.1,
        ASTtol_steps: float = 0.1,
        NameSuffix: str = '',
        verboseMode: bool = True,
        confirmGen: bool = None):
    """
    Parameters
    :param ScriptType: The default is 'MultipleZones'. Can be 'MultipleZones'or
        'mz', or 'SingleZone' or 'sz'.
    :param Outputs: The default is 'Standard'. Can be 'Standard',
        'Simplified' or 'Timestep'.
    :param EnergyPlus_version: The default is 'Ep94'. Can be 'Ep91' or 'Ep94'.
    :param AdapStand: The default is None.(0 = CTE; 1 = EN16798-1; 2 = ASHRAE 55)
    :param CAT: The default is None.(1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 90 = 90% ACCEPT)
    :param ComfMod: The default is None.(0 = Static; 1 = OUT-CTE; 2 = OUT-SEN16798/SASHRAE55; 3 = OUT-AEN16798/AASHRAE55)
    :param HVACmode: The default is None.(0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
    :param VentCtrl: (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit)
    :param VSToffset: Please refer to documentation.
    :param MinOToffset: Please refer to documentation.
    :param MaxWindSpeed: Please refer to documentation.
    :param ASTtol_start: Please refer to documentation.
    :param ASTtol_end_input: Please refer to documentation.
    :param ASTtol_steps: Please refer to documentation.
    :param NameSuffix: Please refer to documentation.

    Exceptions
            DESCRIPTION. EnergyPlus version not supported.
        Only works for EnergyPlus 9.1 (enter Ep91) and
        EnergyPlus 9.4 (enter Ep94) versions.

    :return:
    """
    import accim.sim.accim_Main as main
    import accim.sim.accim_IDFgeneration as IDFgen
    import accim.sim.accim_Base as base
    import accim.sim.accim_Base_EMS as base_EMS
    import accim.sim.accim_SingleZone as sz
    import accim.sim.accim_SingleZone_EMS as sz_EMS
    import accim.sim.accim_MultipleZone as mz
    import accim.sim.accim_MultipleZone_EMS as mz_EMS

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not '_pymod' in file])

    filelist = ([file.split('.idf')[0] for file in filelist])

    objArgsDef = (
        ScriptType is not None,
        Outputs is not None,
        EnergyPlus_version is not None
    )

    fullScriptTypeList = ['MultipleZone',
                          'multiplezone',
                          'mz',
                          'SingleZone',
                          'singlezone',
                          'sz'
    ]

    fullOutputsList = [
        'Standard',
        'standard',
        'Simplified',
        'simplified',
        'Timestep',
        'timestep'
    ]

    fullEPversionsList = [
        'Ep91',
        'ep91',
        'Ep94',
        'ep94'
    ]

    if all(objArgsDef):
        pass
    else:
        ScriptType = input("Enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
        while ScriptType not in fullScriptTypeList:
            ScriptType = input("ScriptType was not correct. Please, enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
        Outputs = input("Enter the Output (Standard, Simplified or Timestep): ")
        while Outputs not in fullOutputsList:
            Outputs = input("Output was not correct. Please, enter the Output (Standard, Simplified or Timestep): ")
        EnergyPlus_version = input("Enter the EnergyPlus version (Ep91 or Ep94): ")
        while EnergyPlus_version not in fullEPversionsList:
            EnergyPlus_version = input("EnergyPlus version was not correct. Please, enter the EnergyPlus version (Ep91 or Ep94): ")
    if verboseMode:
        print('ScriptType is: '+ScriptType)
    if ScriptType not in fullScriptTypeList:
        print('Valid ScriptTypes: ')
        print(fullScriptTypeList)
        raise ValueError(ScriptType + " is not a valid ScriptType. You must choose a ScriptType from the list above.")
    if verboseMode:
        print('Outputs are: '+Outputs)
    if Outputs not in fullOutputsList:
        print('Valid Outputs: ')
        print(fullOutputsList)
        raise ValueError(Outputs + " is not a valid Output. You must choose a Output from the list above.")
    if verboseMode:
        print('EnergyPlus version is: '+EnergyPlus_version)
    if EnergyPlus_version not in fullEPversionsList:
        print('Valid EnergyPlus_version: ')
        print(fullEPversionsList)
        raise ValueError(EnergyPlus_version + " is not a valid EnergyPlus_version. You must choose a EnergyPlus_version from the list above.")

    for file in filelist:
        if verboseMode:
            print('''\n=======================START OF PROCESS=======================\n''')
            print('Starting with file:')
            print(file)
        z = main.accimJob(filename_temp=file, ScriptType=ScriptType, EnergyPlus_version=EnergyPlus_version, verboseMode=verboseMode)

        z.base.setComfFieldsPeople(verboseMode=verboseMode)
        z.base.addOpTempTherm(verboseMode=verboseMode)
        z.base.addBaseSchedules(verboseMode=verboseMode)
        z.base.setAvailSchOn(verboseMode=verboseMode)

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.mz.addMultipleZoneSch(verboseMode=verboseMode)
            z.mz.addCurveObj(verboseMode=verboseMode)
            if EnergyPlus_version.lower() == 'ep94':
                z.mz.addDetHVACobjEp94(verboseMode=verboseMode)
            elif EnergyPlus_version.lower() == 'ep91':
                z.mz.addDetHVACobjEp91(verboseMode=verboseMode)
            z.mz.checkVentIsOn(verboseMode=verboseMode)
            z.mz.addForscriptSchMultipleZone(verboseMode=verboseMode)
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            z.sz.addForscriptSchSingleZone(verboseMode=verboseMode)

        z.base_EMS.addEMSProgramsBase(verboseMode=verboseMode)
        z.base_EMS.addEMSOutputVariableBase(verboseMode=verboseMode)

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.mz_EMS.addGlobVarListMultipleZone(verboseMode=verboseMode)
            z.mz_EMS.addEMSSensorsMultipleZone(verboseMode=verboseMode)
            z.mz_EMS.addEMSActuatorsMultipleZone(verboseMode=verboseMode)
            z.mz_EMS.addEMSProgramsMultipleZone(verboseMode=verboseMode)
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            z.sz_EMS.addGlobVarListSingleZone(verboseMode=verboseMode)
            z.sz_EMS.addEMSSensorsSingleZone(verboseMode=verboseMode)
            z.sz_EMS.addEMSActuatorsSingleZone(verboseMode=verboseMode)
            z.sz_EMS.addEMSProgramsSingleZone(verboseMode=verboseMode)

        z.base_EMS.addEMSPCMBase(verboseMode=verboseMode)

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.mz_EMS.addEMSOutputVariableMultipleZone(verboseMode=verboseMode)
            if Outputs.lower() == 'Simplified'.lower():
                z.base_EMS.addSimplifiedOutputVariables(verboseMode=verboseMode)
            elif Outputs.lower() == 'Standard'.lower():
                z.mz_EMS.addOutputVariablesMultipleZone(verboseMode=verboseMode)
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            if Outputs.lower() == 'Simplified'.lower():
                z.base_EMS.addSimplifiedOutputVariables(verboseMode=verboseMode)
            elif Outputs.lower() == 'Standard'.lower():
                z.sz_EMS.addOutputVariablesSingleZone(verboseMode=verboseMode)
        if Outputs.lower() == 'Timestep'.lower():
            z.base_EMS.addOutputVariablesTimestep(verboseMode=verboseMode)

        z.base.saveaccim(verboseMode=verboseMode)
        if verboseMode:
            print('Ending with file:')
            print(file)
            print('''\n=======================END OF PROCESS=======================\n''')

    # arguments = (
    #     AdapStand is None,
    #     CAT is None,
    #     ComfMod is None,
    #     HVACmode is None,
    #     VentCtrl is None,
    #     VSToffset == [0],
    #     MinOToffset == [50],
    #     MaxWindSpeed == [50],
    #     ASTtol_start == 0.1,
    #     ASTtol_end_input == 0.1,
    #     ASTtol_steps == 0.1
    #     )

    args_needed_mz = (
        AdapStand is not None,
        CAT is not None,
        ComfMod is not None,
        HVACmode is not None,
        VentCtrl is not None,
    )

    args_needed_sz = (
        AdapStand is not None,
        CAT is not None,
        ComfMod is not None,
    )

    if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
        if all(args_needed_mz):
            z.IDFgen.genIDFMultipleZone(
                AdapStand=AdapStand,
                CAT=CAT,
                ComfMod=ComfMod,
                HVACmode=HVACmode,
                VentCtrl=VentCtrl,
                VSToffset=VSToffset,
                MinOToffset=MinOToffset,
                MaxWindSpeed=MaxWindSpeed,
                ASTtol_start=ASTtol_start,
                ASTtol_end_input=ASTtol_end_input,
                ASTtol_steps=ASTtol_steps,
                NameSuffix=NameSuffix,
                verboseMode=verboseMode,
                confirmGen=confirmGen
                )
        else:
            z.IDFgen.inputdataMultipleZone()
            z.IDFgen.genIDFMultipleZone()
    elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
        if all(args_needed_sz):
            z.IDFgen.genIDFSingleZone(
                AdapStand=AdapStand,
                CAT=CAT,
                ComfMod=ComfMod,
                ASTtol_start=ASTtol_start,
                ASTtol_end_input=ASTtol_end_input,
                ASTtol_steps=ASTtol_steps,
                NameSuffix=NameSuffix,
                verboseMode=verboseMode,
                confirmGen=confirmGen
                )
        else:
            z.IDFgen.inputdataSingleZone()
            z.IDFgen.genIDFSingleZone()



import pytest
from accim.sim import accim_Main

def test_genIDFMultipleZone():
    pass
    from eppy.modeleditor import IDF
    from os import listdir
    import numpy

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimJob(
        filename_temp='TestModel_MultipleZone',
        ScriptType='mz',
        EnergyPlus_version='ep94',
        verboseMode=False)

    z.addEMSProgramsMultipleZone(verboseMode=False)
    z.saveaccim(verboseMode=False)

    idf1 = IDF('TestModel_MultipleZone_pymod.idf')
    SetInputData = (
        [program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])

    z.genIDFMultipleZone(
        AdapStand=[1, 2],
        CAT=[1, 80],
        ComfMod=[1, 2],
        HVACmode=[0, 2],
        VentCtrl=[0, 1],
        VSToffset=[0, 1],
        MinOToffset=[5, 10],
        MaxWindSpeed=[5, 10],
        ASTtol_start=0.1,
        ASTtol_end_input=0.2,
        ASTtol_steps=0.1,
        NameSuffix='whatever',
        verboseMode=False,
        confirmGen=True)

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    filelist_pymod = ([file.split('.idf')[0] for file in filelist_pymod])
    print(filelist_pymod)

    AdapStand_List = [1, 2]
    CAT_List = [1, 80]
    ComfMod_List = [1, 80]
    HVACmode_List = [0, 2]
    VentCtrl_List = [0, 1]
    VSToffset_List = [0, 1]
    MinOToffset_List = [5, 10]
    MaxWindSpeed_List = [5, 10]
    ASTtol_value_from = 0.1
    ASTtol_value_to = 0.2
    ASTtol_value_steps = 0.1
    suffix = 'whatever'

    for file in filelist_pymod:
        filename = file

        fname1 = filename + '.idf'
        idf1 = IDF(fname1)

        # print(filename)

        SetInputData = (
        [program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])

        for AdapStand_value in AdapStand_List:
            assert SetInputData[0].Program_Line_1 == 'set AdapStand = ' + repr(AdapStand_value)
            if AdapStand_value == 0:
                assert SetInputData[0].Program_Line_2 == 'set CAT = 1'
                assert SetInputData[0].Program_Line_3 == 'set ComfMod = 0'
                for HVACmode_value in HVACmode_List:
                    assert SetInputData[0].Program_Line_4 == 'set HVACmode = ' + repr(HVACmode_value)
                    if HVACmode_value == 0:
                        for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                         ASTtol_value_steps):
                            assert SetInputData[0].Program_Line_5 == 'set ACSTtol = ' + repr(-ASTtol_value)
                            assert SetInputData[0].Program_Line_6 == 'set AHSTtol = ' + repr(ASTtol_value)
                            outputname = (
                                    filename
                                    + '[AS_CTE'
                                    + '[CA_X'
                                    + '[CM_X'
                                    + '[HM_' + repr(HVACmode_value)
                                    + '[VC_X'
                                    + '[VO_X'
                                    + '[MT_X'
                                    + '[MW_X'
                                    + '[AT_' + repr(ASTtol_value)
                                    + suffix
                                    + '.idf'
                            )
                            assert outputname == filename
                    else:
                        for VentCtrl_value in VentCtrl_List:
                            assert SetInputData[0].Program_Line_5 == 'set VentCtrl = ' + repr(VentCtrl_value)
                            for VSToffset_value in VSToffset_List:
                                assert SetInputData[0].Program_Line_6 == 'set VSToffset = ' + repr(VSToffset_value)
                                for MinOToffset_value in MinOToffset_List:
                                    assert SetInputData[0].Program_Line_7 == 'set MinOToffset = ' + repr(MinOToffset_value)
                                    for MaxWindSpeed_value in MaxWindSpeed_List:
                                        assert SetInputData[0].Program_Line_8 == 'set MaxWindSpeed = ' + repr(
                                            MaxWindSpeed_value)
                                        for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                                         ASTtol_value_steps):
                                            assert SetInputData[0].Program_Line_9 == 'set ACSTtol = ' + repr(-ASTtol_value)
                                            assert SetInputData[0].Program_Line_10 == 'set AHSTtol = ' + repr(ASTtol_value)
                                            outputname = (
                                                    filename
                                                    + '[AS_CTE'
                                                    + '[CA_X'
                                                    + '[CM_X'
                                                    + '[HM_' + repr(HVACmode_value)
                                                    + '[VC_' + repr(VentCtrl_value)
                                                    + '[VO_' + repr(VSToffset_value)
                                                    + '[MT_' + repr(MinOToffset_value)
                                                    + '[MW_' + repr(MaxWindSpeed_value)
                                                    + '[AT_' + repr(ASTtol_value)
                                                    + suffix
                                                    + '.idf'
                                            )
                                            assert outputname == filename
            elif AdapStand_value == 1:
                for CAT_value in CAT_List:
                    if CAT_value not in range(0, 4):
                        continue
                    else:
                        assert SetInputData[0].Program_Line_2 == 'set CAT = ' + repr(CAT_value)
                        for ComfMod_value in ComfMod_List:
                            assert SetInputData[0].Program_Line_3 == 'set ComfMod = ' + repr(ComfMod_value)
                            for HVACmode_value in HVACmode_List:
                                assert SetInputData[0].Program_Line_4 == 'set HVACmode = ' + repr(HVACmode_value)
                                if HVACmode_value == 0:
                                    for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                                     ASTtol_value_steps):
                                        assert SetInputData[0].Program_Line_9 == 'set ACSTtol = ' + repr(-ASTtol_value)
                                        assert SetInputData[0].Program_Line_10 == 'set AHSTtol = ' + repr(ASTtol_value)
                                        outputname = (
                                                filename
                                                + '[AS_EN16798'
                                                + '[CA_' + repr(CAT_value)
                                                + '[CM_' + repr(ComfMod_value)
                                                + '[HM_' + repr(HVACmode_value)
                                                + '[VC_X'
                                                + '[VO_X'
                                                + '[MT_X'
                                                + '[MW_X'
                                                + '[AT_' + repr(ASTtol_value)
                                                + suffix
                                                + '.idf'
                                        )
                                        assert outputname == filename
                                else:
                                    for VentCtrl_value in VentCtrl_List:
                                        assert SetInputData[0].Program_Line_5 == 'set VentCtrl = ' + repr(VentCtrl_value)
                                        for VSToffset_value in VSToffset_List:
                                            assert SetInputData[0].Program_Line_6 == 'set VSToffset = ' + repr(VSToffset_value)
                                            for MinOToffset_value in MinOToffset_List:
                                                assert SetInputData[0].Program_Line_7 == 'set MinOToffset = ' + repr(
                                                    MinOToffset_value)
                                                for MaxWindSpeed_value in MaxWindSpeed_List:
                                                    assert SetInputData[0].Program_Line_8 == 'set MaxWindSpeed = ' + repr(
                                                        MaxWindSpeed_value)
                                                    for ASTtol_value in numpy.arange(ASTtol_value_from,
                                                                                     ASTtol_value_to,
                                                                                     ASTtol_value_steps):
                                                        assert SetInputData[0].Program_Line_9 == 'set ACSTtol = ' + repr(
                                                            -ASTtol_value)
                                                        assert SetInputData[0].Program_Line_10 == 'set AHSTtol = ' + repr(
                                                            ASTtol_value)
                                                        outputname = (
                                                                filename
                                                                + '[AS_EN16798'
                                                                + '[CA_' + repr(CAT_value)
                                                                + '[CM_' + repr(ComfMod_value)
                                                                + '[HM_' + repr(HVACmode_value)
                                                                + '[VC_' + repr(VentCtrl_value)
                                                                + '[VO_' + repr(VSToffset_value)
                                                                + '[MT_' + repr(MinOToffset_value)
                                                                + '[MW_' + repr(MaxWindSpeed_value)
                                                                + '[AT_' + repr(ASTtol_value)
                                                                + suffix
                                                                + '.idf'
                                                        )
                                                        assert outputname == filename
            elif AdapStand_value == 2:
                for CAT_value in CAT_List:
                    if CAT_value not in range(80, 91, 10):
                        continue
                    else:
                        assert SetInputData[0].Program_Line_2 == 'set CAT = ' + repr(CAT_value)
                        for ComfMod_value in ComfMod_List:
                            assert SetInputData[0].Program_Line_3 == 'set ComfMod = ' + repr(ComfMod_value)
                            for HVACmode_value in HVACmode_List:
                                assert SetInputData[0].Program_Line_4 == 'set HVACmode = ' + repr(HVACmode_value)
                                if HVACmode_value == 0:
                                    for ASTtol_value in numpy.arange(ASTtol_value_from, ASTtol_value_to,
                                                                     ASTtol_value_steps):
                                        assert SetInputData[0].Program_Line_9 == 'set ACSTtol = ' + repr(-ASTtol_value)
                                        assert SetInputData[0].Program_Line_10 == 'set AHSTtol = ' + repr(ASTtol_value)
                                        outputname = (
                                                filename
                                                + '[AS_EN16798'
                                                + '[CA_' + repr(CAT_value)
                                                + '[CM_' + repr(ComfMod_value)
                                                + '[HM_' + repr(HVACmode_value)
                                                + '[VC_X'
                                                + '[VO_X'
                                                + '[MT_X'
                                                + '[MW_X'
                                                + '[AT_' + repr(ASTtol_value)
                                                + suffix
                                                + '.idf'
                                        )
                                        assert outputname == filename
                                else:
                                    for VentCtrl_value in VentCtrl_List:
                                        assert SetInputData[0].Program_Line_5 == 'set VentCtrl = ' + repr(VentCtrl_value)
                                        for VSToffset_value in VSToffset_List:
                                            assert SetInputData[0].Program_Line_6 == 'set VSToffset = ' + repr(VSToffset_value)
                                            for MinOToffset_value in MinOToffset_List:
                                                assert SetInputData[0].Program_Line_7 == 'set MinOToffset = ' + repr(
                                                    MinOToffset_value)
                                                for MaxWindSpeed_value in MaxWindSpeed_List:
                                                    assert SetInputData[0].Program_Line_8 == 'set MaxWindSpeed = ' + repr(
                                                        MaxWindSpeed_value)
                                                    for ASTtol_value in numpy.arange(ASTtol_value_from,
                                                                                     ASTtol_value_to,
                                                                                     ASTtol_value_steps):
                                                        assert SetInputData[0].Program_Line_9 == 'set ACSTtol = ' + repr(
                                                            -ASTtol_value)
                                                        assert SetInputData[0].Program_Line_10 == 'set AHSTtol = ' + repr(
                                                            ASTtol_value)
                                                        outputname = (
                                                                filename
                                                                + '[AS_ASHRAE55'
                                                                + '[CA_' + repr(CAT_value)
                                                                + '[CM_' + repr(ComfMod_value)
                                                                + '[HM_' + repr(HVACmode_value)
                                                                + '[VC_' + repr(VentCtrl_value)
                                                                + '[VO_' + repr(VSToffset_value)
                                                                + '[MT_' + repr(MinOToffset_value)
                                                                + '[MW_' + repr(MaxWindSpeed_value)
                                                                + '[AT_' + repr(ASTtol_value)
                                                                + suffix
                                                                + '.idf'
                                                        )
                                                        assert outputname == filename


import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimJob(
        filename_temp='TestModel_MultipleZone',
        ScriptType='mz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_addGlobVarListMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addGlobVarListMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')
    
    gloVar = ([x
               for x
               in idf1.idfobjects['ENERGYMANAGEMENTSYSTEM:GLOBALVARIABLE']])
    assert gloVar[0].Erl_Variable_1_Name == 'ACST'
    assert gloVar[0].Erl_Variable_2_Name == 'AHST'
    assert gloVar[0].Erl_Variable_3_Name == 'ACSTnoTol'
    assert gloVar[0].Erl_Variable_4_Name == 'AHSTnoTol'
    assert gloVar[0].Erl_Variable_5_Name == 'AdapStand'
    assert gloVar[0].Erl_Variable_6_Name == 'ACSTaul'
    assert gloVar[0].Erl_Variable_7_Name == 'ACSTall'
    assert gloVar[0].Erl_Variable_8_Name == 'AHSTaul'
    assert gloVar[0].Erl_Variable_9_Name == 'AHSTall'
    assert gloVar[0].Erl_Variable_10_Name == 'CAT'
    assert gloVar[0].Erl_Variable_11_Name == 'ACSToffset'
    assert gloVar[0].Erl_Variable_12_Name == 'AHSToffset'
    assert gloVar[0].Erl_Variable_13_Name == 'ComfMod'
    assert gloVar[0].Erl_Variable_14_Name == 'ComfTemp'
    assert gloVar[0].Erl_Variable_15_Name == 'ACSTtol'
    assert gloVar[0].Erl_Variable_16_Name == 'AHSTtol'
    assert gloVar[0].Erl_Variable_17_Name == 'VST'
    assert gloVar[0].Erl_Variable_18_Name == 'VSToffset'
    assert gloVar[0].Erl_Variable_19_Name == 'MaxWindSpeed'
    assert gloVar[0].Erl_Variable_20_Name == 'VentCtrl'
    assert gloVar[0].Erl_Variable_21_Name == 'HVACmode'
    assert gloVar[0].Erl_Variable_22_Name == 'MinOutTemp'
    assert gloVar[0].Erl_Variable_23_Name == 'MinOToffset'
    
    zonenames = ([sub.replace(':', '_')
                  for sub
                  in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:GlobalVariable']
                if x.Erl_Variable_1_Name == 'ComfHours_'+zonename])
        assert obj[0].Erl_Variable_1_Name == 'ComfHours_'+zonename
        assert obj[0].Erl_Variable_2_Name == 'DiscomfAppHotHours_'+zonename
        assert obj[0].Erl_Variable_3_Name == 'DiscomfAppColdHours_'+zonename
        assert obj[0].Erl_Variable_4_Name == 'DiscomfNonAppHotHours_'+zonename
        assert obj[0].Erl_Variable_5_Name == 'DiscomfNonAppColdHours_'+zonename
        assert obj[0].Erl_Variable_6_Name == 'ComfHoursNoApp_'+zonename


def test_addEMSSensorsMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSSensorsMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])
    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in idf1.idfobjects['ZONE']])])

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Sensor']
            if x.Name == 'RMOT'])
    assert obj[0].Name == 'RMOT'
    assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == 'People ' + zonenames_orig[0]
    assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Sensor']
            if x.Name == 'PMOT'])
    assert obj[0].Name == 'PMOT'
    assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == 'People ' + zonenames_orig[0]
    assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Thermal Comfort ASHRAE 55 Adaptive Model Running Average Outdoor Air Temperature'

    for i in range(len(zonenames)):
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_OpT'])
        assert obj[0].Name == zonenames[i] + '_OpT'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i]
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Operative Temperature'

        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_CoolCoil'])
        assert obj[0].Name == zonenames[i] + '_CoolCoil'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i] + ' VRF Indoor Unit DX Cooling Coil'
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Cooling Coil Total Cooling Rate'

        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_HeatCoil'])
        assert obj[0].Name == zonenames[i] + '_HeatCoil'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i] + ' VRF Indoor Unit DX Heating Coil'
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Heating Coil Heating Rate'

        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_WindSpeed'])
        assert obj[0].Name == zonenames[i] + '_WindSpeed'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i]
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Outdoor Air Wind Speed'

        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == zonenames[i] + '_OutT'])
        assert obj[0].Name == zonenames[i] + '_OutT'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == zonenames_orig[i]
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Outdoor Air Drybulb Temperature'

        windownamelist_orig = ([window.Name
                                for window
                                in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening']
                                if window.Name.endswith('_Win')])
        windownamelist_orig_split = ([i.split('_') for i in windownamelist_orig])
        windownamelist = ([sub.replace(':', '_')
                           for sub
                           in ([window.Name
                                for window
                                in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening']
                                if window.Name.endswith('_Win')])])


        for i in range(len(windownamelist)):
            obj = ([x
                    for x
                    in idf1.idfobjects['EnergyManagementSystem:Sensor']
                    if x.Name == windownamelist[i]+'_OpT'])
            assert obj[0].Name == windownamelist[i] + '_OpT'
            assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == windownamelist_orig_split[i][0]
            assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Operative Temperature'
            
            obj = ([x
                    for x
                    in idf1.idfobjects['EnergyManagementSystem:Sensor']
                    if x.Name == windownamelist[i] + '_CoolCoil'])
            assert obj[0].Name == windownamelist[i] + '_CoolCoil'
            assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == windownamelist_orig_split[i][0] + ' VRF Indoor Unit DX Cooling Coil'
            assert obj[0].OutputVariable_or_OutputMeter_Name == 'Cooling Coil Total Cooling Rate'

            obj = ([x
                    for x
                    in idf1.idfobjects['EnergyManagementSystem:Sensor']
                    if x.Name == windownamelist[i] + '_HeatCoil'])
            assert obj[0].Name == windownamelist[i] + '_HeatCoil'
            assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == windownamelist_orig_split[i][0] + ' VRF Indoor Unit DX Heating Coil'
            assert obj[0].OutputVariable_or_OutputMeter_Name == 'Heating Coil Heating Rate'

            obj = ([x
                    for x
                    in idf1.idfobjects['EnergyManagementSystem:Sensor']
                    if x.Name == windownamelist[i] + '_WindSpeed'])
            assert obj[0].Name == windownamelist[i] + '_WindSpeed'
            assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == windownamelist_orig_split[i][0]
            assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Outdoor Air Wind Speed'

            obj = ([x
                    for x
                    in idf1.idfobjects['EnergyManagementSystem:Sensor']
                    if x.Name == windownamelist[i] + '_OutT'])
            assert obj[0].Name == windownamelist[i] + '_OutT'
            assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == windownamelist_orig_split[i][0]
            assert obj[0].OutputVariable_or_OutputMeter_Name == 'Zone Outdoor Air Drybulb Temperature'

        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Sensor']
                if x.Name == 'OutT'])
        assert obj[0].Name == 'OutT'
        assert obj[0].OutputVariable_or_OutputMeter_Index_Key_Name == 'Environment'
        assert obj[0].OutputVariable_or_OutputMeter_Name == 'Site Outdoor Air Drybulb Temperature'


def test_addEMSActuatorsMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSActuatorsMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    zonenames = ([sub.replace(':', '_')
                  for sub
                  in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Actuator']
                if x.Name == 'FORSCRIPT_AHST_Sch_' + zonename])
        assert obj[0].Name == 'FORSCRIPT_AHST_Sch_' + zonename
        assert obj[0].Actuated_Component_Unique_Name == 'FORSCRIPT_AHST_' + zonename
        assert obj[0].Actuated_Component_Type == 'Schedule:Compact'
        assert obj[0].Actuated_Component_Control_Type == 'Schedule Value'
        
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Actuator']
                if x.Name == 'FORSCRIPT_ACST_Sch_' + zonename])
        assert obj[0].Name == 'FORSCRIPT_ACST_Sch_' + zonename
        assert obj[0].Actuated_Component_Unique_Name == 'FORSCRIPT_ACST_' + zonename
        assert obj[0].Actuated_Component_Type == 'Schedule:Compact'
        assert obj[0].Actuated_Component_Control_Type == 'Schedule Value'

    windownamelist = ([sub.replace(':', '_') for sub in (
    [window.Name for window in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if
     window.Name.endswith('_Win')])])
    windownamelist_orig = (
    [window.Name for window in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if
     window.Name.endswith('_Win')])

    for i in range(len(windownamelist)):
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Actuator']
                if x.Name == windownamelist[i] + '_VentOpenFact'])
        assert obj[0].Name == windownamelist[i] + '_VentOpenFact'
        assert obj[0].Actuated_Component_Unique_Name == windownamelist_orig[i]
        assert obj[0].Actuated_Component_Type == 'AirFlow Network Window/Door Opening'
        assert obj[0].Actuated_Component_Control_Type == 'Venting Opening Factor'


def test_addEMSProgramsMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSProgramsMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Program']
            if x.Name == 'SetInputData'])
    assert obj[0].Name == 'SetInputData'
    assert obj[0].Program_Line_1 == 'set AdapStand = 1'
    assert obj[0].Program_Line_2 == 'set CAT = 1'
    assert obj[0].Program_Line_3 == 'set ComfMod = 2'
    assert obj[0].Program_Line_4 == 'set HVACmode = 2'
    assert obj[0].Program_Line_5 == 'set VentCtrl = 0'
    assert obj[0].Program_Line_6 == 'set VSToffset = 0'
    assert obj[0].Program_Line_7 == 'set MinOToffset = 7'
    assert obj[0].Program_Line_8 == 'set MaxWindSpeed = 6'
    assert obj[0].Program_Line_9 == 'set ACSTtol = -0.25'
    assert obj[0].Program_Line_10 == 'set AHSTtol = 0.25'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:Program']
            if x.Name == 'SetVST'])
    assert obj[0].Name == 'SetVST'
    assert obj[0].Program_Line_1 == 'set MinOutTemp = AHST - MinOToffset'
    assert obj[0].Program_Line_2 == 'if AdapStand == 0'
    assert obj[0].Program_Line_3 == 'if (CurrentTime < 7)'
    assert obj[0].Program_Line_4 == 'set VST = (ACST+AHST)/2+VSToffset'
    assert obj[0].Program_Line_5 == 'elseif (CurrentTime < 15)'
    assert obj[0].Program_Line_6 == 'set VST = 22.5+VSToffset'
    assert obj[0].Program_Line_7 == 'elseif (CurrentTime < 23)'
    assert obj[0].Program_Line_8 == 'set VST = (ACST+AHST)/2+VSToffset'
    assert obj[0].Program_Line_9 == 'elseif (CurrentTime < 24)'
    assert obj[0].Program_Line_10 == 'set VST = (ACST+AHST)/2+VSToffset'
    assert obj[0].Program_Line_11 == 'endif'
    assert obj[0].Program_Line_12 == 'elseif AdapStand == 1'
    assert obj[0].Program_Line_13 == 'if (RMOT >= AHSTall) && (RMOT <= ACSTaul)'
    assert obj[0].Program_Line_14 == 'set VST = ComfTemp+VSToffset'
    assert obj[0].Program_Line_15 == 'else'
    assert obj[0].Program_Line_16 == 'set VST = (ACST+AHST)/2+VSToffset'
    assert obj[0].Program_Line_17 == 'endif'
    assert obj[0].Program_Line_18 == 'elseif AdapStand == 2'
    assert obj[0].Program_Line_19 == 'if (PMOT >= AHSTall) && (PMOT <= ACSTaul)'
    assert obj[0].Program_Line_20 == 'set VST = ComfTemp+VSToffset'
    assert obj[0].Program_Line_21 == 'else'
    assert obj[0].Program_Line_22 == 'set VST = (ACST+AHST)/2+VSToffset'
    assert obj[0].Program_Line_23 == 'endif'
    assert obj[0].Program_Line_24 == 'endif'
    
    zonenames = ([sub.replace(':', '_')
                  for sub
                  in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Program']
                if x.Name == 'ApplyAST_MultipleZone_' + zonename])
        assert obj[0].Name == 'ApplyAST_MultipleZone_' + zonename
        assert obj[0].Program_Line_1 == 'if (' + zonename + '_OpT>VST)&&(' + zonename + '_OutT < VST)'
        assert obj[0].Program_Line_2 == 'if ' + zonename + '_CoolCoil==0'
        assert obj[0].Program_Line_3 == 'if ' + zonename + '_HeatCoil==0'
        assert obj[0].Program_Line_4 == 'if (' + zonename + '_OpT<ACST)&&(' + zonename + '_OutT>MinOutTemp)'
        assert obj[0].Program_Line_5 == 'if ' + zonename + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_6 == 'set Ventilates_HVACmode2_' + zonename + ' = 1'
        assert obj[0].Program_Line_7 == 'else'
        assert obj[0].Program_Line_8 == 'set Ventilates_HVACmode2_' + zonename + ' = 0'
        assert obj[0].Program_Line_9 == 'endif'
        assert obj[0].Program_Line_10 == 'else'
        assert obj[0].Program_Line_11 == 'set Ventilates_HVACmode2_' + zonename + ' = 0'
        assert obj[0].Program_Line_12 == 'endif'
        assert obj[0].Program_Line_13 == 'else'
        assert obj[0].Program_Line_14 == 'set Ventilates_HVACmode2_' + zonename + ' = 0'
        assert obj[0].Program_Line_15 == 'endif'
        assert obj[0].Program_Line_16 == 'else'
        assert obj[0].Program_Line_17 == 'set Ventilates_HVACmode2_' + zonename + ' = 0'
        assert obj[0].Program_Line_18 == 'endif'
        assert obj[0].Program_Line_19 == 'else'
        assert obj[0].Program_Line_20 == 'set Ventilates_HVACmode2_' + zonename + ' = 0'
        assert obj[0].Program_Line_21 == 'endif'
        assert obj[0].Program_Line_22 == 'if VentCtrl == 0'
        assert obj[0].Program_Line_23 == 'if ' + zonename + '_OutT < ' + zonename + '_OpT'
        assert obj[0].Program_Line_24 == 'if ' + zonename + '_OutT>MinOutTemp'
        assert obj[0].Program_Line_25 == 'if ' + zonename + '_OpT > VST'
        assert obj[0].Program_Line_26 == 'if ' + zonename + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_27 == 'set Ventilates_HVACmode1_' + zonename + ' = 1'
        assert obj[0].Program_Line_28 == 'else'
        assert obj[0].Program_Line_29 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_30 == 'endif'
        assert obj[0].Program_Line_31 == 'else'
        assert obj[0].Program_Line_32 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_33 == 'endif'
        assert obj[0].Program_Line_34 == 'else'
        assert obj[0].Program_Line_35 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_36 == 'endif'
        assert obj[0].Program_Line_37 == 'else'
        assert obj[0].Program_Line_38 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_39 == 'endif'
        assert obj[0].Program_Line_40 == 'elseif VentCtrl == 1'
        assert obj[0].Program_Line_41 == 'if ' + zonename + '_OutT<' + zonename + '_OpT'
        assert obj[0].Program_Line_42 == 'if ' + zonename + '_OutT>MinOutTemp'
        assert obj[0].Program_Line_43 == 'if ' + zonename + '_OpT > ACSTnoTol'
        assert obj[0].Program_Line_44 == 'if ' + zonename + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_45 == 'set Ventilates_HVACmode1_' + zonename + ' = 1'
        assert obj[0].Program_Line_46 == 'else'
        assert obj[0].Program_Line_47 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_48 == 'endif'
        assert obj[0].Program_Line_49 == 'else'
        assert obj[0].Program_Line_50 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_51 == 'endif'
        assert obj[0].Program_Line_52 == 'else'
        assert obj[0].Program_Line_53 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_54 == 'endif'
        assert obj[0].Program_Line_55 == 'else'
        assert obj[0].Program_Line_56 == 'set Ventilates_HVACmode1_' + zonename + ' = 0'
        assert obj[0].Program_Line_57 == 'endif'
        assert obj[0].Program_Line_58 == 'endif'
        assert obj[0].Program_Line_59 == 'if HVACmode == 0'
        assert obj[0].Program_Line_60 == 'set FORSCRIPT_ACST_Sch_' + zonename + ' = ACST'
        assert obj[0].Program_Line_61 == 'set FORSCRIPT_AHST_Sch_' + zonename + ' = AHST'
        assert obj[0].Program_Line_62 == 'elseif HVACmode == 1'
        assert obj[0].Program_Line_63 == 'Set FORSCRIPT_ACST_Sch_' + zonename + ' = 100'
        assert obj[0].Program_Line_64 == 'Set FORSCRIPT_AHST_Sch_' + zonename + ' = -100'
        assert obj[0].Program_Line_65 == 'elseif HVACmode == 2'
        assert obj[0].Program_Line_66 == 'if Ventilates_HVACmode2_' + zonename + ' == 0'
        assert obj[0].Program_Line_67 == 'set FORSCRIPT_ACST_Sch_' + zonename + ' = ACST'
        assert obj[0].Program_Line_68 == 'set FORSCRIPT_AHST_Sch_' + zonename + ' = AHST'
        assert obj[0].Program_Line_69 == 'endif'
        assert obj[0].Program_Line_70 == 'endif'

    windownamelist = ([sub.replace(':', '_') for sub in (
    [window.Name for window in idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if
     window.Name.endswith('_Win')])])

    for windowname in windownamelist:
        obj = ([x
                for x
                in idf1.idfobjects['EnergyManagementSystem:Program']
                if x.Name == 'SetWindowOperation_' + windowname])
        assert obj[0].Name == 'SetWindowOperation_' + windowname
        assert obj[0].Program_Line_1 == 'if (' + windowname + '_OpT>VST)&&(' + windowname + '_OutT < VST)'
        assert obj[0].Program_Line_2 == 'if ' + windowname + '_CoolCoil==0'
        assert obj[0].Program_Line_3 == 'if ' + windowname + '_HeatCoil==0'
        assert obj[0].Program_Line_4 == 'if (' + windowname + '_OpT<ACST)&&(' + windowname + '_OutT>MinOutTemp)'
        assert obj[0].Program_Line_5 == 'if ' + windowname + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_6 == 'set Ventilates_HVACmode2_' + windowname + ' = 1'
        assert obj[0].Program_Line_7 == 'else'
        assert obj[0].Program_Line_8 == 'set Ventilates_HVACmode2_' + windowname + ' = 0'
        assert obj[0].Program_Line_9 == 'endif'
        assert obj[0].Program_Line_10 == 'else'
        assert obj[0].Program_Line_11 == 'set Ventilates_HVACmode2_' + windowname + ' = 0'
        assert obj[0].Program_Line_12 == 'endif'
        assert obj[0].Program_Line_13 == 'else'
        assert obj[0].Program_Line_14 == 'set Ventilates_HVACmode2_' + windowname + ' = 0'
        assert obj[0].Program_Line_15 == 'endif'
        assert obj[0].Program_Line_16 == 'else'
        assert obj[0].Program_Line_17 == 'set Ventilates_HVACmode2_' + windowname + ' = 0'
        assert obj[0].Program_Line_18 == 'endif'
        assert obj[0].Program_Line_19 == 'else'
        assert obj[0].Program_Line_20 == 'set Ventilates_HVACmode2_' + windowname + ' = 0'
        assert obj[0].Program_Line_21 == 'endif'
        assert obj[0].Program_Line_22 == 'if VentCtrl == 0'
        assert obj[0].Program_Line_23 == 'if ' + windowname + '_OutT < ' + windowname + '_OpT'
        assert obj[0].Program_Line_24 == 'if ' + windowname + '_OutT>MinOutTemp'
        assert obj[0].Program_Line_25 == 'if ' + windowname + '_OpT > VST'
        assert obj[0].Program_Line_26 == 'if ' + windowname + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_27 == 'set Ventilates_HVACmode1_' + windowname + ' = 1'
        assert obj[0].Program_Line_28 == 'else'
        assert obj[0].Program_Line_29 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_30 == 'endif'
        assert obj[0].Program_Line_31 == 'else'
        assert obj[0].Program_Line_32 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_33 == 'endif'
        assert obj[0].Program_Line_34 == 'else'
        assert obj[0].Program_Line_35 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_36 == 'endif'
        assert obj[0].Program_Line_37 == 'else'
        assert obj[0].Program_Line_38 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_39 == 'endif'
        assert obj[0].Program_Line_40 == 'elseif VentCtrl == 1'
        assert obj[0].Program_Line_41 == 'if ' + windowname + '_OutT<' + windowname + '_OpT'
        assert obj[0].Program_Line_42 == 'if ' + windowname + '_OutT>MinOutTemp'
        assert obj[0].Program_Line_43 == 'if ' + windowname + '_OpT > ACSTnoTol'
        assert obj[0].Program_Line_44 == 'if ' + windowname + '_WindSpeed <= MaxWindSpeed'
        assert obj[0].Program_Line_45 == 'set Ventilates_HVACmode1_' + windowname + ' = 1'
        assert obj[0].Program_Line_46 == 'else'
        assert obj[0].Program_Line_47 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_48 == 'endif'
        assert obj[0].Program_Line_49 == 'else'
        assert obj[0].Program_Line_50 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_51 == 'endif'
        assert obj[0].Program_Line_52 == 'else'
        assert obj[0].Program_Line_53 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_54 == 'endif'
        assert obj[0].Program_Line_55 == 'else'
        assert obj[0].Program_Line_56 == 'set Ventilates_HVACmode1_' + windowname + ' = 0'
        assert obj[0].Program_Line_57 == 'endif'
        assert obj[0].Program_Line_58 == 'endif'
        assert obj[0].Program_Line_59 == 'if HVACmode == 0'
        assert obj[0].Program_Line_60 == 'set ' + windowname + '_VentOpenFact = 0'
        assert obj[0].Program_Line_61 == 'elseif HVACmode == 1'
        assert obj[0].Program_Line_62 == 'if Ventilates_HVACmode1_' + windowname + ' == 1'
        assert obj[0].Program_Line_63 == 'set ' + windowname + '_VentOpenFact = 1'
        assert obj[0].Program_Line_64 == 'else'
        assert obj[0].Program_Line_65 == 'set ' + windowname + '_VentOpenFact = 0'
        assert obj[0].Program_Line_66 == 'endif'
        assert obj[0].Program_Line_67 == 'elseif HVACmode == 2'
        assert obj[0].Program_Line_68 == 'if Ventilates_HVACmode2_' + windowname + ' == 1'
        assert obj[0].Program_Line_69 == 'set ' + windowname + '_VentOpenFact = 1'
        assert obj[0].Program_Line_70 == 'else'
        assert obj[0].Program_Line_71 == 'set ' + windowname + '_VentOpenFact = 0'
        assert obj[0].Program_Line_72 == 'endif'
        assert obj[0].Program_Line_73 == 'endif'


def test_addEMSOutputVariableMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSOutputVariableMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:OutputVariable']
            if x.Name == 'Ventilation Setpoint Temperature'])
    assert obj[0].Name == 'Ventilation Setpoint Temperature'
    assert obj[0].EMS_Variable_Name == 'VST'
    assert obj[0].Type_of_Data_in_Variable == 'Averaged'
    assert obj[0].Update_Frequency == 'ZoneTimestep'
    assert obj[0].EMS_Program_or_Subroutine_Name == ''
    assert obj[0].Units == 'C'

    obj = ([x
            for x
            in idf1.idfobjects['EnergyManagementSystem:OutputVariable']
            if x.Name == 'Minimum Outdoor Temperature for MultipleZone ventilation'])
    assert obj[0].Name == 'Minimum Outdoor Temperature for MultipleZone ventilation'
    assert obj[0].EMS_Variable_Name == 'MinOutTemp'
    assert obj[0].Type_of_Data_in_Variable == 'Averaged'
    assert obj[0].Update_Frequency == 'ZoneTimestep'
    assert obj[0].EMS_Program_or_Subroutine_Name == ''
    assert obj[0].Units == 'C'


def test_addOutputVariablesMultipleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addOutputVariablesMultipleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')

    outputvariablelist = ([outputvariable.Name
                           for outputvariable
                           in idf1.idfobjects['EnergyManagementSystem:OutputVariable']])

    for outputvariable in outputvariablelist:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Variable_Name == outputvariable])
        assert obj[0].Key_Value == '*'
        assert obj[0].Variable_Name == outputvariable
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''
        
    addittionaloutputs = [
        'Zone Thermostat Operative Temperature',
        'Zone Thermal Comfort CEN 15251 Adaptive Model Running Average Outdoor Air Temperature',
        'Cooling Coil Total Cooling Rate',
        'Heating Coil Heating Rate',
        'Facility Total HVAC Electric Demand Power',
        'Facility Total HVAC Electricity Demand Rate',
        'VRF Heat Pump Cooling Electricity Energy',
        'VRF Heat Pump Heating Electricity Energy',
        'AFN Surface Venting Window or Door Opening Factor',
        'AFN Zone Infiltration Air Change Rate',
        'AFN Zone Infiltration Volume'
        ]

    for addittionaloutput in addittionaloutputs:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Variable_Name == addittionaloutput])
        assert obj[0].Key_Value == '*'
        assert obj[0].Variable_Name == addittionaloutput
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''
        
    zonenames = ([sub.replace(':', '_')
                  for sub
                  in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for zonename in zonenames:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == 'FORSCRIPT_AHST_' + zonename])
        assert obj[0].Key_Value == 'FORSCRIPT_AHST_'+zonename
        assert obj[0].Variable_Name == 'Schedule Value'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''
        
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == 'FORSCRIPT_ACST_' + zonename])
        assert obj[0].Key_Value == 'FORSCRIPT_ACST_'+zonename
        assert obj[0].Variable_Name == 'Schedule Value'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''

    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])
    
    for zonename in zonenames_orig:
        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == zonename
                and x.Variable_Name == 'Zone Operative Temperature'])
        assert obj[0].Key_Value == zonename
        assert obj[0].Variable_Name == 'Zone Operative Temperature'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''

        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == zonename+' VRF Indoor Unit DX Cooling Coil'
                and x.Variable_Name == 'Cooling Coil Total Cooling Rate'])
        assert obj[0].Key_Value == zonename+' VRF Indoor Unit DX Cooling Coil'
        assert obj[0].Variable_Name == 'Cooling Coil Total Cooling Rate'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''

        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == zonename + ' VRF Indoor Unit DX Heating Coil'
                and x.Variable_Name == 'Heating Coil Heating Rate'])
        assert obj[0].Key_Value == zonename+' VRF Indoor Unit DX Heating Coil'
        assert obj[0].Variable_Name == 'Heating Coil Heating Rate'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''

        obj = ([x
                for x
                in idf1.idfobjects['Output:Variable']
                if x.Key_Value == zonename
                and x.Variable_Name == 'Zone Outdoor Air Wind Speed'])
        assert obj[0].Key_Value == zonename
        assert obj[0].Variable_Name == 'Zone Outdoor Air Wind Speed'
        assert obj[0].Reporting_Frequency == 'Hourly'
        assert obj[0].Schedule_Name == ''



class Table():
    import os
    import pandas as pd
    from pathlib import Path
    import numpy as np
    from collections import defaultdict
    import copy
    pass

    def __init__(self):
        import os
        import pandas as pd
        from pathlib import Path
        import numpy as np
        from collections import defaultdict
        import copy
        self.source_files = sorted(Path(os.getcwd()).glob('*.csv'))

        self.source_files_sample = sorted(Path(os.getcwd()).glob('*.csv'))[0]
        df_sample = pd.DataFrame(pd.read_csv(self.source_files_sample))
        OpTempColumn = [i for i in df_sample.columns if 'Operative Temperature [C](Hourly)' in i]
        self.block_zone_list_colon = [i.split(' ')[0] for i in OpTempColumn]
        self.block_zone_list_colon = list(dict.fromkeys(self.block_zone_list_colon))
        self.block_zone_list_colon = list(dict.fromkeys(self.block_zone_list_colon))
        self.block_zone_list_colon = [i[:-5] for i in self.block_zone_list_colon]
        self.block_zone_list_underscore = [i.replace(':', '_') for i in self.block_zone_list_colon]
        self.block_list = [i.split(':')[0] for i in self.block_zone_list_colon]
        self.block_list = list(dict.fromkeys(self.block_list))
        self.allcols = df_sample.columns

        self.cities = ['Canfranc',
                       'Bilbao',
                       'Huesca',
                       'Coruna',
                       'Sevilla',
                       'Valencia',
                       'Zaragoza',
                       'Fuerteventura']
        self.years = ['2015',
                      '2016',
                      '2017',
                      '2018']

        print(self.source_files)
        print(os.getcwd())
        summed_dataframes = []
        for file in self.source_files:
            self.df = pd.DataFrame(pd.read_csv(file))
            self.df['source'] = file.name
            self.vrfCols = [col for col in self.df.columns if 'VRF' in col]
            self.vrfCols = [col for col in self.vrfCols if 'OUTDOOR' in col]
            self.vrfCols = [col for col in self.vrfCols if 'Hourly' in col]
            self.vrfCols_renamed = [i.split('_')[1].rstrip() for i in self.vrfCols]
            self.vrfCols_renamed = [i.split(' ') for i in self.vrfCols_renamed]
            self.vrfCols_renamed = [[i for i in nested if
                                     i != 'Heat' and
                                     i != 'Pump' and
                                     i != 'Electricity' and
                                     i != 'Energy' and
                                     i != '[J](Hourly)'] for nested in self.vrfCols_renamed]
            self.block_zone_list_vrf = [i[0] for i in self.vrfCols_renamed]
            self.block_zone_list_vrf = list(dict.fromkeys(self.block_zone_list_vrf))
            self.block_list = [i.split(':')[0] for i in self.block_zone_list_vrf]
            self.block_list = list(dict.fromkeys(self.block_list))
            self.vrfCols_renamed = ['_'.join(i) for i in self.vrfCols_renamed]
            vofCols = [i for i in self.df.columns if 'Surface Venting Window or Door Opening Factor [](Hourly)' in i]
            VOFsample = []
            for block_zone in self.block_zone_list_colon:
                sublist = []
                for i in range(len(vofCols)):
                    if block_zone in vofCols[i]:
                        sublist.append(vofCols[i])
                VOFsample.append(sublist)
            VOFdef = [i[0] for i in VOFsample]
            self.vofCols_renamed = ['Ventilation Hours '+i.split('_')[0] for i in VOFdef]

            listresult = [
                'comfHoursNoAppCols',
                'comfHoursCols',
                'disAppHotHourCols',
                'disAppColdHourCols',
                'disNonAppHotHourCols',
                'disNonAppColdHourCols',
                ]
            originalcols = defaultdict(list)
            for i in range(len(self.allcols)):
                for block_zone in self.block_zone_list_underscore:
                    if f'EMS:Comfortable Hours_No Applicability_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[0]].append(self.allcols[i])
                    elif f'EMS:Comfortable Hours_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[1]].append(self.allcols[i])
                    elif f'EMS:Discomfortable Applicable Hot Hours_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[2]].append(self.allcols[i])
                    elif f'EMS:Discomfortable Applicable Cold Hours_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[3]].append(self.allcols[i])
                    elif f'EMS:Discomfortable Non Applicable Hot Hours_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[4]].append(self.allcols[i])
                    elif f'EMS:Discomfortable Non Applicable Cold Hours_{block_zone} (summed) [H](Hourly)'.lower() in self.allcols[i].lower():
                        originalcols[listresult[5]].append(self.allcols[i])
            renamedcols = copy.deepcopy(originalcols)
            for block_zone in self.block_zone_list_underscore:
                for i in range(len(listresult)):
                    for j in range(len(originalcols[listresult[i]])):
                        if f'EMS:Comfortable Hours_No Applicability_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Comfortable Hours_No Applicability {block_zone}'
                        elif f'EMS:Comfortable Hours_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Comfortable Hours {block_zone}'
                        elif f'EMS:Discomfortable Applicable Hot Hours_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Discomfortable Applicable Hot Hours {block_zone}'
                        elif f'EMS:Discomfortable Applicable Cold Hours_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Discomfortable Applicable Cold Hours {block_zone}'
                        elif f'EMS:Discomfortable Non Applicable Hot Hours_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Discomfortable Non Applicable Hot Hours {block_zone}'
                        elif f'EMS:Discomfortable Non Applicable Cold Hours_{block_zone} (summed) [H](Hourly)'.lower() in originalcols[listresult[i]][j].lower():
                            renamedcols[listresult[i]][j] = f'Discomfortable Non Applicable Cold Hours {block_zone}'
            self.df = self.df.loc[:, self.vrfCols +
                                  VOFdef +
                                  originalcols['comfHoursNoAppCols'] +
                                  originalcols['comfHoursCols'] +
                                  originalcols['disAppHotHourCols'] +
                                  originalcols['disAppColdHourCols'] +
                                  originalcols['disNonAppHotHourCols'] +
                                  originalcols['disNonAppColdHourCols']
                                  ]
            colslist = [*self.vrfCols_renamed,
                        *self.vofCols_renamed,
                        *renamedcols['comfHoursNoAppCols'],
                        *renamedcols['comfHoursCols'],
                        *renamedcols['disAppHotHourCols'],
                        *renamedcols['disAppColdHourCols'],
                        *renamedcols['disNonAppHotHourCols'],
                        *renamedcols['disNonAppColdHourCols']]

            self.df.columns = colslist
            self.df['source'] = file
            self.df = self.df.groupby('source').agg('sum')
            self.df['source_temp'] = file
            summed_dataframes.append(self.df)
        self.summed_tot_df = pd.concat(summed_dataframes)
        print(self.summed_tot_df)
        self.summed_tot_df[['model',
                            'standard',
                            'cat',
                            'comfMod',
                            'hvacMode',
                            'ventControl',
                            'ventOffset',
                            'minOutTemp',
                            'maxWindSpeed',
                            'astTol',
                            'city_year']] = self.summed_tot_df['source_temp'].astype(str).str.extract('.+\\\\(.+_pymod)\[(AS_\w+)\[(CA_\w)\[(CM_\w)\[(HM_\w)\[(VC_\w)\[(VO_\w.\w)\[(MT_\w\w.\w)\[(MW_\w\w.\w)\[(AT_\w.\w)\[(.+).csv', expand=True)
        del self.summed_tot_df['source_temp']
        self.summed_tot_df['model'] = self.summed_tot_df['model'].str[:-6]
        self.summed_tot_df['standard'] = self.summed_tot_df['standard'].str[3:]
        self.summed_tot_df['cat'] = self.summed_tot_df['cat'].str[3:]
        self.summed_tot_df['comfMod'] = self.summed_tot_df['comfMod'].str[3:]
        self.summed_tot_df['hvacMode'] = self.summed_tot_df['hvacMode'].str[3:]
        self.summed_tot_df['ventControl'] = self.summed_tot_df['ventControl'].str[3:]
        self.summed_tot_df['ventOffset'] = self.summed_tot_df['ventOffset'].str[3:]
        self.summed_tot_df['minOutTemp'] = self.summed_tot_df['minOutTemp'].str[3:]
        self.summed_tot_df['maxWindSpeed'] = self.summed_tot_df['maxWindSpeed'].str[3:]
        self.summed_tot_df['astTol'] = self.summed_tot_df['astTol'].str[3:]
        for year in self.years:
            for city in self.cities:
                self.summed_tot_df['city_year'] = np.where(
                    (self.summed_tot_df['city_year'].str.contains(city, case=False)) &
                    (self.summed_tot_df['city_year'].str.contains(year, case=False)),
                    city+'_'+year,
                    self.summed_tot_df['city_year'])
        self.summed_tot_df[['city', 'year']] = self.summed_tot_df['city_year'].str.extract(r'(.+)_(.+)', expand=True)
        del self.summed_tot_df['city_year']

    def EnergyConsumptionTable(self):
        """Generate the Energy consumption table for csv files in\
        current working directory."""
        import os
        multiindex = ['model',
                      'standard',
                      'cat',
                      'comfMod',
                      'hvacMode',
                      'ventControl',
                      'ventOffset',
                      'minOutTemp',
                      'maxWindSpeed',
                      'astTol',
                      'city',
                      'year']

        self.EnergyConsumpTable = self.summed_tot_df.loc[:, multiindex + self.vrfCols_renamed]
        self.EnergyConsumpTable = self.EnergyConsumpTable.apply(lambda x: x/3600000 if x.name in self.vrfCols_renamed else x)
        for block in self.block_list:
            print(block)
            self.EnergyConsumpTable[f'{block}_Total'] = self.EnergyConsumpTable[[i for i in self.EnergyConsumpTable.columns if block in i]].sum(axis=1)
            print('f{block}_Total')
            self.EnergyConsumpTable[f'{block}_Heating'] = self.EnergyConsumpTable[[i for i in self.EnergyConsumpTable.columns if block in i and '_Heating' in i]].sum(axis=1)
            print(f'{block}_Heating')
            self.EnergyConsumpTable[f'{block}_Cooling'] = self.EnergyConsumpTable[[i for i in self.EnergyConsumpTable.columns if block in i and '_Cooling' in i]].sum(axis=1)
            print(f'{block}_Cooling')
        for block_zone in self.block_zone_list_vrf:
            print(block_zone)
            self.EnergyConsumpTable[f'{block_zone}_Total'] = self.EnergyConsumpTable[f'{block_zone}_Heating'] + self.EnergyConsumpTable[f'{block_zone}_Cooling']
        self.EnergyConsumpTable['Total Heating'] = self.EnergyConsumpTable[[i for i in self.EnergyConsumpTable.columns if 'Heating' in i and 'VRF' not in i]].sum(axis=1)
        self.EnergyConsumpTable['Total Cooling'] = self.EnergyConsumpTable[[i for i in self.EnergyConsumpTable.columns if 'Cooling' in i and 'VRF' not in i]].sum(axis=1)
        self.EnergyConsumpTable['Total'] = self.EnergyConsumpTable['Total Cooling'] + self.EnergyConsumpTable['Total Heating']
        block_zone_cols = [i for i in self.EnergyConsumpTable.columns if 'VRF' in i]
        block_cols = []
        for i in range(len(self.EnergyConsumpTable.columns)):
            for block in self.block_list:
                if block in self.EnergyConsumpTable.columns[i] and 'VRF' not in self.EnergyConsumpTable.columns[i]:
                    block_cols.append(self.EnergyConsumpTable.columns[i])
        block_cols
        total_cols = ['Total Heating', 'Total Cooling', 'Total']
        colsorder = multiindex+total_cols
        self.EnergyConsumpTable = self.EnergyConsumpTable.loc[:, colsorder]
        self.EnergyConsumpTable = self.EnergyConsumpTable.reindex(columns=colsorder)
        return self.EnergyConsumpTable
        self.EnergyConsumpTable.to_excel(excel_writer=os.getcwd() +
                                         '/Energy consumption table.xlsx',
                                         sheet_name='Total Energy Consumption',
                                         float_format="%.2f",
                                         engine='xlsxwriter')
        
    def EnergyConsumpHorBarPlot(self):
        """Generate the Energy consumption horizontal barplot\
            from the EnergyConsumptionTable."""
        #intenta buscar el archivo os.getcwd() +'/Energy consumption table.xlsx' o self.EnergyConsumptionTable, y si no existe, self.EnergyConsumptionTable() y partimos de self.EnergyConsumptionTable
        pass
